from nose.config import Config
from optparse import OptionParser
from unittest import TestCase

class Tests(TestCase):

    def setUp(self):
        from nose_fixes.plugin import Plugin
        self.plugin = Plugin()
        
    def test_enabled(self):
        self.assertTrue(self.plugin.enabled)

    def test_suite_defined(self):
        self.assertEqual(self.plugin.name, 'nose_fixes')

    def _parse_args(self, *args):
        parser = OptionParser()
        self.plugin.addOptions(parser)
        options, args = parser.parse_args(list(args))
        self.plugin.configure(options, Config())
        
    def test_suite_name_default(self):
        self._parse_args()
        self.assertEqual(self.plugin.test_suite_func, 'test_suite')
    
    def test_suite_name_override(self):
        self._parse_args('--test-suite-func=foo')
        self.assertEqual(self.plugin.test_suite_func, 'foo')



from imp import new_module
from nose.case import Test
from unittest import TestCase

class Tests(TestCase):

    def setUp(self):
        from nose_fixes.plugin import Plugin
        plugin = Plugin()
        plugin.test_suite_func = 'test_suite'
        self.l = plugin.prepareTestLoader(None)
        
    def test_suite(self):

        suite = object()
        def test_suite():
            # return a suite
            return suite
        
        mod = new_module('with_test_suite')
        mod.test_suite = test_suite

        self.assertTrue(self.l.loadTestsFromModule(mod) is suite)

    def test_no_suite(self):
        def test_sweet():
            # a tester of sweets ;-)
            assert True!=False
        
        mod = new_module('without_test_suite')
        mod.test_sweet = test_sweet

        suite = self.l.loadTestsFromModule(mod)

        tests = [t for t in suite]
        self.assertEqual(len(tests), 1)
        test = tests[0]
        self.assertTrue(isinstance(test, Test))
        self.assertEqual(repr(test), 'Test(without_test_suite.test_sweet)')



from nose.plugins import Plugin as NosePlugin
from nose.loader import TestLoader

class BetterLoader(TestLoader):

    def __init__(self, test_suite_func):
        super(BetterLoader, self).__init__()
        self.test_suite_func = test_suite_func
        
    def loadTestsFromModule(self, module, path=None, discovered=False):
        suite_func = getattr(module, self.test_suite_func, None)
        if suite_func is not None:
            return suite_func()
        return super(BetterLoader, self).loadTestsFromModule(
            module, path, discovered
            )
    
class Plugin(NosePlugin):

    name = 'nose_fixes'

    enabled = True
    
    def options(self, parser, env):
        parser.add_option("--test-suite-func", action="store",
                          dest="test_suite_func",
                          default='test_suite',
                          help="A function in modules that will return a TestSuite. "
                               "Defaults to 'test_suite'.")
    
    def configure(self, options, config):
        self.test_suite_func = options.test_suite_func
        
    def prepareTestLoader(self, loader):
        return BetterLoader(self.test_suite_func)
